import { Body, Controller, Get, Post, UseGuards } from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import { ApiBody, ApiOperation, ApiTags } from '@nestjs/swagger';
import { Recaptcha } from '@nestlab/google-recaptcha';
import { Serealize } from '../interceptors/serialize.interceptor';
import { AuthService } from './auth.service';
import { LoginAdminDto } from './dto/input/login-admin.dto';
import { LoginOperadorDto } from './dto/input/login-operador.dto';
import { LoginUsuarioDto } from './dto/input/login-usuario.dto';
import { AuthTokenOutputDto } from './dto/output/token.dto';

@Controller('auth')
@ApiTags('auth')
export class AuthController {
  constructor(private authService: AuthService) {}

  @Serealize(AuthTokenOutputDto)
  @Recaptcha()
  @Post('login-admin')
  @ApiOperation({
    description: 'Endpoint utilizado para el login del panel admin.',
  })
  @ApiBody({
    description: 'Todas las variables son obligatorias.',
    examples: {
      ejemplo: {
        value: {
          operador: '',
          password: '',
        },
      },
    },
  })
  loginAdmin(@Body() body: LoginAdminDto) {
    return this.authService.loginAdmin(body.operador, body.password);
  }

  @Serealize(AuthTokenOutputDto)
  @Recaptcha()
  @Post('login-operador')
  @ApiOperation({
    description: 'Endpoint utilizado para el login del panel del operador.',
  })
  @ApiBody({
    description: 'Todas las variables son obligatorias.',
    examples: {
      ejemplo: {
        value: {
          id_modulo: 1,
          operador: '',
          password: '',
        },
      },
    },
  })
  loginOperador(@Body() body: LoginOperadorDto) {
    return this.authService.loginOperador(
      body.id_modulo,
      body.operador,
      body.password,
    );
  }

  @Serealize(AuthTokenOutputDto)
  @Recaptcha()
  @Post('login-usuario')
  @ApiOperation({
    description: 'Endpoint utilizado para el login del usuario.',
  })
  @ApiBody({
    description: 'Todas las variables son obligatorias.',
    examples: { ejemplo: { value: { password: '', usuario: '' } } },
  })
  loginUsuario(@Body() body: LoginUsuarioDto) {
    return this.authService.loginUsuario(body.usuario, body.password);
  }

  @Get('validar-token')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Endpoint utilizado para validar un token.' })
  validarToken() {
    return { valido: true };
  }
}
