import {
  BadRequestException,
  Injectable,
  UnauthorizedException,
} from '@nestjs/common';
import { JwtService } from '@nestjs/jwt';
import { Operador } from '../operador/entity/operador.entity';
import { BcryptService } from '../bcrypt/bcrypt.service';
import { ModuloService } from '../modulo/modulo.service';
import { OperadorService } from '../operador/operador.service';
import { UsuarioService } from '../usuario/usuario.service';
import { JwtPayloadAdmin } from './dto/jwt-payload-admin';
import { JwtPayloadOperador } from './dto/jwt-payload-operador';
import { JwtPayloadUsuario } from './dto/jwt-payload-usuario';

@Injectable()
export class AuthService {
  constructor(
    private bcryptService: BcryptService,
    private jwtService: JwtService,
    private moduloService: ModuloService,
    private operadorService: OperadorService,
    private usuarioService: UsuarioService,
  ) {}

  loginAdmin(admin: string, password: string) {
    return this.operadorService.findInfoOperadorAdmin(admin).then((admin) => {
      this.validacionBasica(admin, password);

      const JwtPayload: JwtPayloadAdmin = {
        Operador: {
          id_operador: admin.id_operador,
          operador: admin.operador,
          nombre: admin.nombre,
          institucion: {
            id_institucion: admin.institucion.id_institucion,
          },
          tipoUsuario: {
            id_tipo_usuario: admin.tipoUsuario.id_tipo_usuario,
          },
        },
      };
      return { token: this.jwtService.sign(JwtPayload) };
    });
  }

  async loginOperador(id_modulo: number, operador: string, password: string) {
    const modulo = await this.moduloService.findInfoModuloById(id_modulo);

    return this.operadorService
      .findInfoOperador(modulo.institucion, operador)
      .then((operador) => {
        this.validacionBasica(operador, password);

        const JwtPayload: JwtPayloadOperador = {
          Operador: {
            id_operador: operador.id_operador,
            operador: operador.operador,
            nombre: operador.nombre,
            institucion: {
              id_institucion: operador.institucion.id_institucion,
            },
            tipoUsuario: {
              id_tipo_usuario: operador.tipoUsuario.id_tipo_usuario,
            },
          },
        };

        if (operador.tipoUsuario.id_tipo_usuario === 4)
          JwtPayload.Modulo.id_modulo = modulo.id_modulo;
        return { token: this.jwtService.sign(JwtPayload) };
      });
  }

  loginUsuario(usuario: string, password: string) {
    return this.usuarioService
      .informacionUsuarioByUsuario(usuario)
      .then((data) => {
        if (!data.password)
          throw new BadRequestException('Este usuario no ha sido registrado.');
        if (
          !data.usuario ||
          !this.bcryptService.comparar(password, data.password)
        )
          throw new BadRequestException(
            'Usuario y/o password incorrectos, ingresa unas credenciales válidas.',
          );
        return { token: this.jwtService.sign(data.usuario) };
      });
  }

  private validacionBasica(operador: Operador, password: string) {
    if (!operador || !this.bcryptService.comparar(password, operador.password))
      throw new BadRequestException(
        'Usuario y/o password incorrectos, ingresa unas credenciales válidas.',
      );
    if (!operador.activo)
      throw new UnauthorizedException('Esta cuenta se encuentra desactivada.');
  }
}
