import { Injectable, NotFoundException } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Not, Repository } from 'typeorm';
import { Carrera } from '../institucion-carrera/entity/carrera.entity';
import { Institucion } from '../institucion/entity/institucion.entity';
import { InstitucionCarrera } from './entity/institucion-carrera.entity';
import { InstitucionService } from '../institucion/institucion.service';

@Injectable()
export class InstitucionCarreraService {
  constructor(
    @InjectRepository(Carrera) private carreraRepository: Repository<Carrera>,
    @InjectRepository(InstitucionCarrera)
    private institucionCarreraRepository: Repository<InstitucionCarrera>,
    private institucionService: InstitucionService,
  ) {}

  findAllByIdInstitucion(id_institucion: number) {
    return this.institucionService
      .findInfoInstitucionById(id_institucion)
      .then((institucion) =>
        this.institucionCarreraRepository.find({
          where: {
            carrera: { id_carrera: Not(1) },
            institucion,
          },
          order: { carrera: { carrera: 'ASC' } },
        }),
      );
  }

  findById(id_institucion_carrera: number) {
    return this.institucionCarreraRepository
      .findOne({ where: { id_institucion_carrera } })
      .then((institucionCarrera) => {
        if (!institucionCarrera)
          throw new NotFoundException('No existe este id institución carrera.');
        return institucionCarrera;
      });
  }

  async findByIdInstitucionIdCarrera(
    id_institucion: number | Institucion,
    id_carrera: number | Carrera,
  ) {
    const carrera =
      typeof id_carrera === 'number'
        ? await this.findCarreraByIdCarrera(id_carrera)
        : id_carrera;
    const institucion =
      typeof id_institucion === 'number'
        ? await this.institucionService.findInfoInstitucionById(id_institucion)
        : id_institucion;

    return this.institucionCarreraRepository.findOne({
      where: { carrera, institucion },
    });
  }

  findCarreraByIdCarrera(id_carrera: number) {
    return this.carreraRepository
      .findOne({ where: { id_carrera } })
      .then((carrera) => {
        if (!carrera) throw new NotFoundException('No existe este id carrera.');
        return carrera;
      });
  }

  findCarreraByCarrera(carrera: string, validarNoExiste = true) {
    return this.carreraRepository
      .findOne({ where: { carrera } })
      .then((carrera) => {
        if (validarNoExiste && !carrera)
          throw new NotFoundException('No existe esta carrera.');
        return carrera;
      });
  }

  findInstitucionProfesor(id_institucion: number) {
    return this.institucionService
      .findInfoInstitucionById(id_institucion)
      .then((institucion) =>
        this.institucionCarreraRepository.findOne({
          where: { carrera: { id_carrera: 1 }, institucion },
        }),
      );
  }
}
